@@ -1,8 +1,9 @@
-__version__ = "0.3.6"
+__version__ = "0.3.8"
 
 import warnings
-from ._static_view import jupyter_displayable_output
+from ._static_view import jupyter_displayable_output, insight
 from ._utilities import merge_rgb
+from ._context import nop
 
 class _SliceViewer():
     def __init__(self,
@@ -12,7 +13,9 @@ class _SliceViewer():
                  display_width: int = None,
                  display_height: int = None,
                  continuous_update: bool = False,
-                 slider_text: str = "Slice"
+                 slider_text: str = "Slice",
+                 zoom_factor:float = 1.0,
+                 zoom_spline_order:int = 0
                  ):
         import ipywidgets
         from ._image_widget import ImageWidget
@@ -24,9 +27,9 @@ class _SliceViewer():
             slice_number = int(image.shape[axis] / 2)
 
         if len(image.shape) <= 2:
-            self.view = ImageWidget(image)
+            self.view = ImageWidget(image, zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order)
         else:
-            self.view = ImageWidget(np.take(image, slice_number, axis=axis))
+            self.view = ImageWidget(np.take(image, slice_number, axis=axis), zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order)
         if display_width is not None:
             self.view.width = display_width
         if display_height is not None:
@@ -42,6 +45,7 @@ class _SliceViewer():
                 continuous_update=continuous_update,
                 description=slider_text,
             )
+            # widgets.link((sliders1, 'value'), (slider2, 'value'))
 
         # event handler when the user changed something:
         def configuration_updated(event):
@@ -66,7 +70,9 @@ def slice(
         display_width : int = None,
         display_height : int = None,
         continuous_update:bool=False,
-        slider_text:str="Slice"
+        slider_text:str="Slice",
+        zoom_factor:float = 1.0,
+        zoom_spline_order:int = 0
 ):
     """Shows an image with a slider to go through a stack.
 
@@ -79,11 +85,15 @@ def slice(
     axis : int, optional
         Axis in case we are slicing a stack
     display_width : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     display_height : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     continuous_update : bool, optional
         Update the image while dragging the mouse, default: False
+    zoom_factor: float, optional
+        Allows showing the image larger (> 1) or smaller (<1)
+    zoom_spline_order: int, optional
+        Spline order used for interpolation (default=0, nearest-neighbor)
 
     Returns
     -------
@@ -97,7 +107,9 @@ def slice(
         display_width,
         display_height,
         continuous_update,
-        slider_text
+        slider_text,
+        zoom_factor=zoom_factor,
+        zoom_spline_order=zoom_spline_order
     )
     view = viewer.view
     slice_slider = viewer.slice_slider
@@ -116,7 +128,9 @@ def curtain(
         display_width: int = None,
         display_height: int = None,
         continuous_update: bool = False,
-        alpha: float = 1
+        alpha: float = 1,
+        zoom_factor:float = 1.0,
+        zoom_spline_order:int = 0
 ):
     """Show two images and allow with a slider to show either the one or the other image.
 
@@ -131,13 +145,17 @@ def curtain(
     axis : int, optional
         Axis in case we are slicing a stack
     display_width : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     display_height : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     continuous_update : bool, optional
         Update the image while dragging the mouse, default: False
     alpha: float, optional
         sets the transperancy of the curtain
+    zoom_factor: float, optional
+        Allows showing the image larger (> 1) or smaller (<1)
+    zoom_spline_order: int, optional
+        Spline order used for interpolation (default=0, nearest-neighbor)
         
     Returns
     -------
@@ -173,9 +191,9 @@ def curtain(
     )
 
     if len(image.shape) <= 2:
-        view = ImageWidget(image)
+        view = ImageWidget(image, zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order)
     else:
-        view = ImageWidget(np.take(image, slice_number, axis=axis))
+        view = ImageWidget(np.take(image, slice_number, axis=axis), zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order)
     if display_width is not None:
         view.width = display_width
     if display_height is not None:
@@ -215,7 +233,9 @@ def orthogonal(
         image,
         display_width : int = None,
         display_height : int = None,
-        continuous_update:bool=False
+        continuous_update:bool=False,
+        zoom_factor:float = 1.0,
+        zoom_spline_order:int = 0
 ):
     """Show three viewers slicing the image stack in Z,Y and X.
 
@@ -224,11 +244,15 @@ def orthogonal(
     image : image
         Image to be displayed
     display_width : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     display_height : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     continuous_update : bool, optional
         Update the image while dragging the mouse, default: False
+    zoom_factor: float, optional
+        Allows showing the image larger (> 1) or smaller (<1)
+    zoom_spline_order: int, optional
+        Spline order used for interpolation (default=0, nearest-neighbor)
 
     Returns
     -------
@@ -241,9 +265,9 @@ def orthogonal(
     import ipywidgets
 
     return ipywidgets.HBox([
-        slice(image, axis=0, slider_text="Z", display_width=display_width, display_height=display_height, continuous_update=continuous_update),
-        slice(image, axis=1, slider_text="Y", display_width=display_width, display_height=display_height, continuous_update=continuous_update),
-        slice(image, axis=2, slider_text="X", display_width=display_width, display_height=display_height, continuous_update=continuous_update),
+        slice(image, axis=0, slider_text="Z", display_width=display_width, display_height=display_height, continuous_update=continuous_update, zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order),
+        slice(image, axis=1, slider_text="Y", display_width=display_width, display_height=display_height, continuous_update=continuous_update, zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order),
+        slice(image, axis=2, slider_text="X", display_width=display_width, display_height=display_height, continuous_update=continuous_update, zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order),
     ])
 
 
@@ -255,7 +279,9 @@ def side_by_side(
         display_width: int = None,
         display_height: int = None,
         continuous_update: bool = False,
-        slider_text: str = "Slice"
+        slider_text: str = "Slice",
+        zoom_factor:float = 1.0,
+        zoom_spline_order:int = 0
 ):
     """Shows two images in magenta and green plus a third with their colocalization / overlap view and
     a slider to go through a stack.
@@ -271,11 +297,15 @@ def side_by_side(
     axis : int, optional
         Axis in case we are slicing a stack
     display_width : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     display_height : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     continuous_update : bool, optional
         Update the image while dragging the mouse, default: False
+    zoom_factor: float, optional
+        Allows showing the image larger (> 1) or smaller (<1)
+    zoom_spline_order: int, optional
+        Spline order used for interpolation (default=0, nearest-neighbor)
 
     Returns
     -------
@@ -295,9 +325,9 @@ def side_by_side(
         slice_image = np.take(image1, slice_number, axis=axis)
 
     zeros_image = np.zeros(slice_image.shape)
-    view1 = ImageWidget(slice_image)
-    view2 = ImageWidget(slice_image)
-    view3 = ImageWidget(slice_image)
+    view1 = ImageWidget(slice_image, zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order)
+    view2 = ImageWidget(slice_image, zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order)
+    view3 = ImageWidget(slice_image, zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order)
 
     if display_width is not None:
         view1.display = display_width
@@ -372,9 +402,12 @@ def side_by_side(
 
 
 def interact(func,
-             image,
+             image = None,
              *args,
              continuous_update: bool = False,
+             context:dict = None,
+             zoom_factor:float = 1.0,
+             zoom_spline_order:int = 0,
              **kwargs):
     """Takes a function which has an image as first parameter and additional parameters.
     It will build a user interface consisting of sliders for numeric parameters and parameters
@@ -383,17 +416,38 @@ def interact(func,
     Parameters
     ----------
     func : function
-    image : Image
-    continuous_update : bool
+    image : Image, optional
+        If not provided, context must be provided instead.
     args
+    continuous_update : bool, optioonal
+        Update the image while dragging the mouse, default: False
+    context:dict, optional
+        A dictionary of (name:image), allows showing a pulldown of available images.
+    zoom_factor: float, optional
+        Allows showing the image larger (> 1) or smaller (<1)
+    zoom_spline_order: int, optional
+        Spline order used for interpolation (default=0, nearest-neighbor)
+    context:dict
+        dictionary of name, value pairs that can be selected from pulldowns, e.g.: globals()
     kwargs
 
     """
     import inspect
     import ipywidgets
+    from ._utilities import parameter_is_image_parameter
+    from ._context import Context
 
     exposable_parameters = []
     footprint_parameters = []
+    image_parameters = []
+
+    if context is not None:
+        context = Context(context)
+
+    image_passed = image is not None
+    if context is not None and image is None:
+        image = next(iter(context._images.values()))
+
 
     sig = inspect.signature(func)
     for key in sig.parameters.keys():
@@ -401,37 +455,54 @@ def interact(func,
         default_value = 0
         if isinstance(sig.parameters[key].default, int) or isinstance(sig.parameters[key].default, float):
             default_value = sig.parameters[key].default
+        min_value, max_value, step = guess_range(key, sig.parameters[key].annotation)
+
+        if min_value is not None:
+            int_slider = ipywidgets.IntSlider
+            float_slider = ipywidgets.FloatSlider
+        else:
+            int_slider = ipywidgets.IntText
+            float_slider = ipywidgets.FloatText
 
         if sig.parameters[key].annotation is int:
-            default_value = ipywidgets.IntSlider(min=0, max=20, step=1, value=default_value, continuous_update=continuous_update)
+            default_value = int_slider(min=min_value, max=max_value, step=step, value=default_value, continuous_update=continuous_update)
             exposable = True
-        elif sig.parameters[key].annotation is float:
-            default_value = ipywidgets.FloatSlider(min=-20, max=20, step=1, value=default_value, continuous_update=continuous_update)
-            exposable = True
-        elif key == 'sigma' or key == 'radius':
-            default_value = ipywidgets.FloatSlider(min=-20, max=20, step=1, value=default_value, continuous_update=continuous_update)
+        elif sig.parameters[key].annotation is float or 'sigma' in key or 'radius' in key:
+            default_value = float_slider(min=min_value, max=max_value, step=step, value=default_value, continuous_update=continuous_update)
             exposable = True
         elif key.startswith("is_") or sig.parameters[key].annotation is bool:
             default_value = ipywidgets.Checkbox(value=default_value)
             exposable = True
         elif key == 'footprint' or key == 'selem' or key == 'structuring_element':
             footprint_parameters.append(key)
-            default_value = ipywidgets.IntSlider(min=0, max=20, step=1, value=default_value, continuous_update=continuous_update)
+            default_value = ipywidgets.IntSlider(min=min_value, max=max_value, step=step, value=default_value, continuous_update=continuous_update)
             exposable = True
+        elif parameter_is_image_parameter(sig.parameters[key]) and "destination" not in key and key != "out"  and key != "output":
+            if context is not None:
+                image_parameters.append(key)
+                default_value = ipywidgets.Dropdown(
+                    options=list(context._images.keys())
+                    #options=[(k, v) for k, v in context._images.items()],
+                )
+                exposable = True
 
         if exposable:
             if key in kwargs.keys():
                 default_value = kwargs[key]
             exposable_parameters.append(inspect.Parameter(key, inspect.Parameter.KEYWORD_ONLY, default=default_value))
 
-    viewer = _SliceViewer(image)
+    viewer = _SliceViewer(image, zoom_factor=zoom_factor, zoom_spline_order=zoom_spline_order)
+    if viewer.slice_slider is not None:
+        viewer.slice_slider.continuous_update=continuous_update
     command_label = ipywidgets.Label(value=func.__name__ + "()")
 
     from skimage import morphology
 
     def worker_function(*otherargs, **kwargs):
 
-        command = func.__name__ + "(..."
+        command = func.__name__ + "("
+        if image_passed:
+            command = command + "..."
 
         for key in [e.name for e in exposable_parameters]:
 
@@ -442,19 +513,23 @@ def interact(func,
                 elif len(image.shape) == 3:
                     command = command + ", " + key + "=ball(" + str(kwargs[key]) + ")"
                     kwargs[key] = morphology.ball(kwargs[key])
+            elif key in image_parameters:
+                command = command + ", " + key + "=" + str(kwargs[key])
+                kwargs[key] = context._images[kwargs[key]]
             else:
                 command = command + ", " + key + "=" + str(kwargs[key])
         command = command + ")"
-        command_label.value = command
+        command_label.value = command.replace("(,", "(")
 
-        viewer.image = func(image, *args, **kwargs)
+        if image_passed:
+            viewer.image = func(image, *args, **kwargs)
+        else:
+            viewer.image = func(*args, **kwargs)
+        if viewer.slice_slider is not None:
+            viewer.slice_slider.max = viewer.image.shape[0] - 1
         viewer.configuration_updated(None)
 
 
-
-    import ipywidgets
-
-
     worker_function.__signature__ = inspect.Signature(exposable_parameters)
     ipywidgets.interact(worker_function)
 
@@ -475,6 +550,18 @@ def interact(func,
             command_label
         ])
 
+def guess_range(name, annotation):
+    if name == 'footprint' or name == 'selem' or name == 'structuring_element':
+        return 0, 100, 1
+    if 'sigma' in name:
+        return 0, 10, 1
+    if 'radius' in name:
+        return 0, 100, 1
+    if 'factor' in name:
+        return 0, 100, 1
+    if name == 'angle' or "degrees" in name:
+        return 0, 360, 15
+    return None, None, None
 
 def picker(
         image,
@@ -482,7 +569,9 @@ def picker(
         display_width: int = None,
         display_height: int = None,
         continuous_update: bool = False,
-        slider_text: str = "Slice"
+        slider_text: str = "Slice",
+        zoom_factor:float = 1.0,
+        zoom_spline_order:int = 0
 ):
     """Shows an image with a slider to go through a stack plus a label with the current mouse position and intensity at that position.
 
@@ -493,11 +582,15 @@ def picker(
     slice_number : int, optional
         Slice-position in the stack
     display_width : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     display_height : int, optional
-        Size of the displayed image in pixels
+        This parameter is obsolete. Use zoom_factor instead
     continuous_update : bool, optional
         Update the image while dragging the mouse, default: False
+    zoom_factor: float, optional
+        Allows showing the image larger (> 1) or smaller (<1)
+    zoom_spline_order: int, optional
+        Spline order used for interpolation (default=0, nearest-neighbor)
 
     Returns
     -------
@@ -510,7 +603,9 @@ def picker(
                           display_width=display_width,
                           display_height=display_height,
                           continuous_update=continuous_update,
-                          slider_text=slider_text
+                          slider_text=slider_text,
+                          zoom_factor=zoom_factor,
+                          zoom_spline_order=zoom_spline_order
                           )
     view = viewer.view
     slice_slider = viewer.slice_slider
@@ -520,8 +615,8 @@ def picker(
     event_handler = Event(source=view, watched_events=['mousemove'])
 
     def update_display(event):
-        relative_position_x = event['relativeX']
-        relative_position_y = event['relativeY']
+        relative_position_x = event['relativeX'] / zoom_factor
+        relative_position_y = event['relativeY'] / zoom_factor
         absolute_position_x = int(relative_position_x)
         absolute_position_y = int(relative_position_y)
 
@@ -544,3 +639,6 @@ def picker(
 def _no_resize(widget):
     import ipywidgets
     return ipywidgets.HBox([ipywidgets.VBox([widget])])
+
+
+
